# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
import cv2
import numpy as np
from pathlib import Path


def read(img_path: Path):
    """
    读图片-兼容图片路径包含中文
    :param img_path:
    :return: np.ndarray
    """
    img = cv2.imdecode(np.fromfile(str(img_path), dtype=np.uint8), cv2.IMREAD_COLOR)
    return img


def write(image: np.ndarray, img_write_path: Path):
    """
    写图片-兼容图片路径包含中文
    :param image:
    :param img_write_path:
    :return:
    """
    cv2.imencode(img_write_path.suffix, image[:, :, ::-1])[1].tofile(str(img_write_path))



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function:
"""
安装
pip install pdf2image

安装依赖库poppler
linux:
sudo apt-get install poppler-utils

windows:
https://blog.alivate.com.au/poppler-windows/
将路径 E:/.../venv/poppler-0.68.0/bin 添加到系统环境变量path中后重启
"""
from pdf2image import convert_from_path
from pathlib import Path
from PIL import Image
Image.MAX_IMAGE_PIXELS = None


def export_image_from_pdf(pdf_dir: Path, dist_dir: Path, dpi=300, quality=100):
    """
    从pdf中导出图片
    :param pdf_dir: pdf目录
    :param dist_dir: 导出文件目录
    :param dpi: 导出图片的分辨率
    :param quality: 保存图片的质量
    :return:
    """
    total_pdf = 0
    total_img = 0
    for child_dir in pdf_dir.iterdir():
        pdfs = []
        if child_dir.is_file():
            pdfs.append(child_dir)
        else:
            for pdf_file in child_dir.glob("*.pdf"):
                pdfs.append(pdf_file)
        for pdf_file in pdfs:
            if pdf_file.suffix not in [".pdf", ".PDF"]:
                continue
            total_pdf += 1
            save_img_d = dist_dir.joinpath(pdf_file.stem)
            if save_img_d.exists():
                continue
            print("正在导出......", pdf_file.stem)
            images = convert_from_path(pdf_file, dpi=dpi, fmt='jpg')
            if not save_img_d.exists():
                save_img_d.mkdir(parents=True)
            for image in images:
                save_img = save_img_d.joinpath("{0}.jpg".format(images.index(image)+1))
                image.save(save_img, quality=quality)
                total_img += 1
                print(total_img)
        print("pdf", total_pdf)
        print("img", total_img)


if __name__ == "__main__":
    pass




# -*- coding:utf-8 -*-
# Author:      zhousf
# File:        interceptor_util.py
# Description:  AOP拦截器
import functools


def intercept(before=None, after=None):
    """
    拦截器
    :param before: 过滤器
    :param after: 过滤器
    def before(*args):
        if args[0] == "0":
            return True, '拦截'
        return False, '不拦截'
    :return:
    """
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*chain, **kw):
            if before is not None:
                result_before = before(chain)
                if isinstance(result_before, tuple):
                    need_intercept, msg = result_before
                else:
                    need_intercept = result_before
                    msg = "interceptor"
                if need_intercept:
                    return msg
                result = func(msg, **kw)
            else:
                result = func(chain, **kw)
            return result if after is None else after(result)
        return wrapper

    return decorator




# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function: 环境配置
import os
from enum import Enum, unique


@unique
class Environment(Enum):
    Development = "Development"
    Testing = "Testing"
    Production = "Production"


class CoreConfig(object):
    DEBUG = False
    JSON_AS_ASCII = False
    JSON_SORT_KEYS = False
    JSONIFY_MIMETYPE = 'application/json;charset=utf-8'
    SQLALCHEMY_TRACK_MODIFICATIONS = True
    PROJECT_DIR = os.path.dirname(__file__)
    PID_FILE = "{0}/pid.txt".format(PROJECT_DIR)
    GPU_ASSIGNED = "0"
    HOST = "0.0.0.0"
    PORT = 8090
    LOG_BUSINESS_DIR = None
    LOG_SERVICE_DIR = None
    LOG_GUNICORN_DIR = None
    ALLOWED_FILE_TYPE = ["jpg", "JPG", "jpeg", "JPEG", "png", "PNG", "bmp", "BMP"]
    VISIT_JSON_FILE = "{0}/visit-list.json".format(PROJECT_DIR)
    CHECK_PERMISSION = True

    @staticmethod
    def has_pid_file(obj):
        return hasattr(obj, "PID_FILE")

    @staticmethod
    def read_txt(txt_file):
        """
        读取txt文档并返回list
        :param txt_file:
        :return:
        """
        data = []
        if not os.path.exists(txt_file):
            return data
        with open(txt_file) as f:
            lines = f.readlines()
            for line in lines:
                line = line.replace("\n", "")
                if line.strip() != "":
                    data.append(line)
        return data

    def shutdown(self):
        if hasattr(self, "PID_FILE"):
            if not os.path.exists(self.PID_FILE):
                return
            process = self.read_txt(self.PID_FILE)
            print(process)
            for pid in process:
                if os.name == 'nt':
                    cmd = 'taskkill /pid ' + str(pid) + ' /f'
                    try:
                        os.system(cmd)
                        print(pid, "killed")
                    except Exception as e:
                        print(e)
                elif os.name == 'posix':
                    try:
                        command = "kill -9 {0}".format(pid)
                        os.system(command)
                        print(pid, "killed")
                    except Exception as e:
                        print(e)
            os.remove(self.PID_FILE)


def assign_env(cf, env, log_dir=None):
    """
    指定环境并创建日志目录
    :param cf: 配置文件
    :param env: 环境
    :param log_dir: 默认空，日志目录默认在工程根目录下创建
    """
    # 指定环境
    if env == Environment.Production:
        env_instance = cf.Production()
        env_instance.ENVIRONMENT = Environment.Production
    elif env == Environment.Testing:
        env_instance = cf.Testing()
        env_instance.ENVIRONMENT = Environment.Testing
    else:
        env_instance = cf.Development()
        env_instance.ENVIRONMENT = Environment.Development
    if env_instance.PROJECT_DIR is None:
        # 在模块的根目录下创建日志目录
        log_root_dir = "{0}/log".format(os.path.dirname(__file__))
    else:
        # 在工程的根目录下创建日志目录
        log_root_dir = "{0}/log".format(CoreConfig.PROJECT_DIR)
    # 创建日志目录
    if log_dir is not None:
        log_root_dir = log_dir
    log_app_dir = "{0}".format(log_root_dir)
    env_instance.LOG_BUSINESS_DIR = "{0}/business".format(log_app_dir)
    env_instance.LOG_SERVICE_DIR = "{0}/service".format(log_app_dir)
    env_instance.LOG_GUNICORN_DIR = "{0}/gunicorn".format(log_app_dir)
    if not os.path.exists(env_instance.LOG_BUSINESS_DIR):
        os.makedirs(env_instance.LOG_BUSINESS_DIR)
    if not os.path.exists(env_instance.LOG_SERVICE_DIR):
        os.makedirs(env_instance.LOG_SERVICE_DIR)
    if not os.path.exists(env_instance.LOG_GUNICORN_DIR):
        os.makedirs(env_instance.LOG_GUNICORN_DIR)
    return env_instance


class Development(CoreConfig):
    """
    开发环境
    """
    HOST = "0.0.0.0"
    PORT = 5005
    DEBUG = True


class Testing(CoreConfig):
    """
    测试环境
    """
    HOST = "0.0.0.0"
    PORT = 5005
    DEBUG = False


class Production(CoreConfig):
    """
    生产环境
    """
    HOST = "0.0.0.0"
    PORT = 5005
    DEBUG = False







# -*- coding: utf-8 -*-
# @Author  : zhousf-a
# @Function:
import openpyxl
from pathlib import Path
from zhousflib.util import re_util


def unmerge_and_fill_cells(excel_file: Path, merged_cell_rate=2/3, delete_duplicates=True, tmp_excel: Path = None,
                           target_sheet_name=None) -> Path:
    """
    拆分合并单元格并填充有效值
    :param excel_file:
    :param merged_cell_rate: 合并单元格的数量占列数的比例，若大于该比例则拆分该合并单元格
    :param delete_duplicates: 是否对拆分合并单元格的结果去重
    :param tmp_excel: 临时文件，若为空则会更新源文件
    :param target_sheet_name: None第一张表
    :return:
    """
    wb = openpyxl.load_workbook(str(excel_file))
    contain_merge_cells = False
    for sheet_name in wb.sheetnames:
        if target_sheet_name:
            if target_sheet_name != sheet_name:
                continue
        worksheet = wb[sheet_name]
        all_merged_cell_ranges = list(worksheet.merged_cells.ranges)
        rows_deal = {}
        """
        拆分合并单元格
        """
        for merged_cell_range in all_merged_cell_ranges:
            merged_cell = merged_cell_range.start_cell
            worksheet.unmerge_cells(range_string=merged_cell_range.coord)
            start, end = merged_cell_range.coord.split(":")
            start = int(re_util.get_digit_char(start))
            end = int(re_util.get_digit_char(end))
            if (start, end) not in rows_deal:
                rows_deal[(start, end)] = 1
            else:
                rows_deal[(start, end)] += 1

            for row_index, col_index in merged_cell_range.cells:
                cell = worksheet.cell(row=row_index, column=col_index)
                cell.value = merged_cell.value
        """
        找到符合拆分合并单元格条件的单元格rows
        """
        need_fill = []
        for i in rows_deal:
            column_count = rows_deal.get(i)
            if column_count >= merged_cell_rate * worksheet.max_column:
                need_fill.append(i)
                contain_merge_cells = True
        """
        拆分合并单元格后，对空单元格赋予有效值
        """
        for cells in worksheet.iter_rows():
            for cell in cells:
                row = cell.row
                column = cell.column
                for fill in need_fill:
                    if row == fill[0]:
                        next_cell = worksheet.cell(fill[1], column)
                        if not cell.value and next_cell.value:
                            cell.value = next_cell.value
                        if cell.value and not next_cell.value:
                            next_cell.value = cell.value
        """
        拆分合并单元格后会有重复的两条，这里去重一下
        """
        if delete_duplicates:
            for fill in need_fill:
                data = []
                for row_cells in worksheet.iter_rows(min_row=min(fill), max_row=max(fill)):
                    data.append([cell.value for cell in row_cells])
                if len(data) == 2:
                    if data[0] == data[1]:
                        worksheet.delete_rows(idx=fill[1])
    if tmp_excel:
        wb.save(str(tmp_excel))
        wb.close()
        return tmp_excel
    else:
        if contain_merge_cells:
            wb.save(str(excel_file))
        wb.close()
        return excel_file



# -*- coding:utf-8 -*-
# Author:      zhousf
# File:        json_util.py
# Description:
import json


def class_to_json(class_obj):
    """
    对象转成json字典
    :param class_obj: 对象
    :return: json字典字符串
    """
    return json.dumps(class_obj, default=lambda obj: obj.__dict__, sort_keys=True, indent=4)


def dict_to_json(dic):
    """
    字典转成json
    :param dic: 字典
    :return:
    """
    return json.dumps(dic, sort_keys=True, indent=4)


def json_str_to_class(json_str, object_hook):
    """
    json字符串转成对象
    :param json_str: json字符串
    :param object_hook: 回调函数
    :return:
    """
    return json.loads(json_str, object_hook=object_hook)


def write_dict_into_json_file(a_dict, json_file):
    """
    将字典写入json文件中
    :param a_dict: 字典
    :param json_file: json文件
    :return:
    """
    with open(json_file, 'w') as f:
        json.dump(a_dict, f, ensure_ascii=False, sort_keys=True, indent=4, separators=(',', ': '))


def write_obj_into_json_file(obj, json_file):
    """
    将对象写入json文件中
    :param obj: 对象
    :param json_file: json文件
    :return:
    """
    a_dict = class_to_json(obj)
    if not isinstance(a_dict, dict):
        a_dict = eval(a_dict)
    write_dict_into_json_file(a_dict, json_file)


def load_obj_from_json_file(obj, json_file):
    """
    json文件转成对象
    :param obj:
    :param json_file:
    :return:
    """
    with open(json_file, 'r') as f:
        content = f.read()
        if content is not None and content.strip() != '':
            obj.__dict__ = json.loads(s=content)
            return obj
    return None


def load_dict_from_json_file(json_file):
    """
    json文件转成字典
    :param json_file:
    :return: dict
    """
    with open(json_file, 'r') as f:
        content = f.read()
        if content is not None and content.strip() != '':
            return json.loads(s=content)
    return None


def sort(json_or_dict):
    """
    排序：对key排序
    :param json_or_dict: {'后门壳（左）': ['刮擦'], '前门壳（左）': ['撕裂', '刮擦']}
    :return: {'前门壳（左）': ['刮擦', '撕裂'], '后门壳（左）': ['刮擦']}
    """
    json_or_dict = json.loads(json.dumps(json_or_dict, sort_keys=True, ensure_ascii=False))
    for k in json_or_dict:
        if isinstance(json_or_dict[k], list):
            # 字典不排序 by guanning
            if json_or_dict[k] and isinstance(json_or_dict[k][0], dict):
                for index in range(0, len(json_or_dict[k])):
                    json_or_dict[k][index] = sorted(json_or_dict[k][index].items(), key=lambda x: x[0], reverse=True)
                #按排序好的字典key(字典已经变成元组)的第一个 将list排序
                json_or_dict[k] = sorted(json_or_dict[k], key=lambda x: x[0])

            else:
                json_or_dict[k] = sorted(json_or_dict[k])
    return json_or_dict


if __name__ == '__main__':
    d = {'far': '', 'middle': '/media/ubuntu/b8f80802-d95a-41c3-b157-6f4e34967425/workspace/AI_TEST/damage/2018071802/middle.jpg', 'near': '', 'code': '0000', 'message': 'success', 'result': ''}
    write_dict_into_json_file(d, 't.json')






# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function: 图像分类数据制作
import shutil
import imghdr
import numpy as np
from pathlib import Path
from zhousflib.util import list_util
"""
一般情况下，在加载预训练模型的情况下，每个类别包括 10-20 张图像即可保证基本的分类效果；
不加载预训练模型的情况下，每个类别需要至少包含 100-200 张图像以保证基本的分类效果。
训练集、验证集、测试集的类别都要全，不然会给训练带来麻烦
"""


def fetch_available_cls_folder(img_dir: Path):
    """
    删除空目录
    :param img_dir:
    :return:
    """
    for folder in img_dir.iterdir():
        cls = [i for i in folder.rglob("*.*")]
        if len(cls) == 0:
            print(folder)
            shutil.rmtree(folder)


def train_test_split(image_dir: Path, val_size=0.2, test_size=0.2, shuffle=True, every_cls_count_limit=20):
    """
    训练集、验证集、测试集划分
    :param image_dir: 图片目录
    :param val_size: 验证集占比
    :param test_size: 测试集占比
    :param shuffle: 打乱数据集顺序
    :param every_cls_count_limit: 每个类别最低的图片数量，若低于则随机复制
    :return:
    """
    train_txt_file = image_dir.parent.joinpath("train_list.txt")
    val_txt_file = image_dir.parent.joinpath("val_list.txt")
    test_txt_file = image_dir.parent.joinpath("test_list.txt")
    label_list_file = image_dir.parent.joinpath("label_list.txt")
    images = []
    label_list = []
    # 标签文件
    for folder in image_dir.rglob("*.*"):
        if folder.parent.name not in label_list:
            label_list.append(folder.parent.name)
    if not label_list_file.exists():
        with label_list_file.open("w", encoding="utf-8") as f:
            for i, d in enumerate(label_list):
                f.write("{0} {1}\n".format(i, d))
    label_files = {}
    # 遍历所有图片文件
    for folder in image_dir.rglob("*.*"):
        if not folder.is_file():
            continue
        if not imghdr.what(folder):
            continue
        file = "{0}/{1}/{2} {3}\n".format(folder.parent.parent.name, folder.parent.name, folder.name, label_list.index(folder.parent.name))
        if folder.parent.name not in label_files:
            label_files[folder.parent.name] = [file]
        else:
            label_files[folder.parent.name].append(file)
        print(file)
        images.append(file)
    # 随机复制低于every_cls_count_limit的类别
    for label in label_files:
        label_count = len(label_files.get(label)) if label_files.get(label) else 0
        if label_count < every_cls_count_limit:
            for item in list_util.random_choices(label_files.get(label), choose_k=abs(every_cls_count_limit-label_count)):
                images.append(item)
    # 打乱顺序
    if shuffle:
        state = np.random.get_state()
        np.random.shuffle(images)
        np.random.set_state(state)
    dataset_val = []
    dataset_test = []
    split_index = 0
    if 1 > val_size > 0:
        split_index = int(len(images) * val_size)
        dataset_val = images[:split_index]
    if 1 > test_size > 0:
        start = split_index
        split_index += int(len(images) * test_size)
        dataset_test = images[start:split_index]
    dataset_train = images[split_index:]
    # 训练集
    if len(dataset_train) > 0:
        with train_txt_file.open("w", encoding="utf-8") as f:
            for d in dataset_train:
                f.write(d)
    # 验证集
    if len(dataset_val) > 0:
        with val_txt_file.open("w", encoding="utf-8") as f:
            for d in dataset_val:
                f.write(d)
    # 测试集集
    if len(dataset_test) > 0:
        with test_txt_file.open("w", encoding="utf-8") as f:
            for d in dataset_test:
                f.write(d)


def data_statistics(image_dir: Path):
    """
    数据分布统计
    :param image_dir: 数据集目录
    :return:
    """
    label_files = {}
    # 遍历所有图片文件
    for folder in image_dir.rglob("*.*"):
        if not folder.is_file():
            continue
        if not imghdr.what(folder):
            continue
        if folder.parent.name not in label_files:
            label_files[folder.parent.name] = [folder]
        else:
            label_files[folder.parent.name].append(folder)
    for label in label_files:
        print("{0}: {1}".format(label, len(label_files.get(label))))


if __name__ == "__main__":

    # train_test_split(image_dir=Path(r"C:\Users\zhousf-a\Desktop\steel_id\images"), val_size=0.2, test_size=0,
    #                  shuffle=False)
    data_statistics(image_dir=Path(r"C:\Users\zhousf-a\Desktop\steel_id\images"))
    pass








# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function: 计算器
import re


def cal(calc_formula: str) -> str:
    """
    公式计算，保持原有格式计算数值的项
    :param calc_formula:
    :return:
    "64.7*2+(70.6+332.9)/2*2+13*2+L"  ->  "558.9+L"
    "2*L+64.7-8"  ->  "2*L+56.7"
    "(63~64)*2+10*2"  ->  "(63~64)*2+20"
    """
    calc_formula = calc_formula.replace("（", "(").replace("）", ")").replace(" ", "")
    target_list = re.split(r"(?=[-+])", calc_formula)
    number_list = []
    not_number_list = []
    number_is_first = False
    # 过滤
    for i in range(len(target_list) - 1, -1, -1):
        target = target_list[i]
        # 过滤空字符串
        if not str(target).strip():
            target_list.pop(i)
        # 过滤非有效值
        if target in ["+", "-", "*", "/"]:
            target_list.pop(i)
    # 获取数值列表和非数值列表
    for i in range(0, len(target_list)):
        target = target_list[i]
        if len(str(target).split("|")) > 1:
            not_number_list.append(target)
            continue
        try:
            eval(target)
            number_list.append(target)
            if i == 0:
                number_is_first = True
        except Exception as e:
            not_number_list.append(target)
    number_str = ["{0}".format(number) for number in number_list]
    number_str = "".join(number_str)
    union_symbol = ""
    if len(number_str) > 0:
        if number_str[0] in ["*", "/"]:
            union_symbol = number_str[0]
            number_str = number_str[1:]
    number_str_cal_result = eval("".join(number_str)) if len(number_str) > 0 else ""
    not_number_str = ["{0}".format(not_number) for not_number in not_number_list]
    if number_is_first:
        result_union = str(number_str_cal_result) + union_symbol + "".join(not_number_str)
    else:
        if not number_str_cal_result and not union_symbol and len(str(number_str_cal_result)) > 0:
            print(number_str_cal_result, type(number_str_cal_result), len(str(number_str_cal_result)))
            if number_str_cal_result > 0:
                union_symbol = "+"
        result_union = "".join(not_number_str) + union_symbol + str(number_str_cal_result)
    return result_union


if __name__ == "__main__":
    # print(cal("64.7*2+(70.6+332.9)/2*2+13*2+L"))
    # print(cal("2*L+64.7-8"))
    # print(cal("(63~64)*2+10*2"))
    # print(cal("64.7*2+339.7*2+13*2"))
    # print(cal("2+/2+1000"))
    # print(cal(".+250+320+200+200+200"))
    # print(cal("*250+200"))
    # print(cal("+752+2+"))
    # print(cal("+05+120"))
    # print(cal("100*2/4"))
    print(cal("167.6+167.6+183+25+30+30|30+63+25+25"))
    pass




# -*- coding:utf-8 -*-
# Author:  zhousf-a
# Description: pdf表格还原工具
# pip install pdfplumber
import pdfplumber
import pandas as pd


def pdf_to_excel(pdf_file, save_file):
    """
    读取pdf表格并保存到excel文件中
    :param pdf_file:
    :param save_file:
    :return:
    """
    pdf_data = pdfplumber.open(str(pdf_file))
    result_df = pd.DataFrame()
    for page in pdf_data.pages:
        table = page.extract_table()
        print(table)
        df_detail = pd.DataFrame(table[1:], columns=table[0])
        result_df = pd.concat([df_detail, result_df], ignore_index=True)
    # result_df.dropna(axis=1, how='all', inplace=True)
    result_df.to_excel(excel_writer=str(save_file), index=False, encoding='utf-8')


def read_pdf(pdf_file):
    """
    读取pdf表格
    """
    pdf_data = pdfplumber.open(str(pdf_file))
    tables = []
    texts = []
    table_settings = {
        "explicit_vertical_lines": [10],
    }
    for page in pdf_data.pages:
        # image = page.to_image()
        # image.draw_rects(page.extract_words())
        # image.show()
        table = page.extract_table(table_settings)
        text = page.extract_text()
        if table:
            tables.extend(table)
        if isinstance(text, list):
            texts.extend(text)
            continue
        texts.append(text)
    return tables, texts, pdf_data


if __name__ == "__main__":
    pass



# -*- coding:utf-8 -*-
# Author:  zhousf
# Description: coco 数据集合并
import shutil
import json
from pathlib import Path


def merge_dataset_coco(dataset_dirs: list, dist_dir: Path, img_index=0):
    """
    coco数据集合并
    :param dataset_dirs: ["batch1/dataset_coco/test", "batch2/dataset_coco/test"]
    :param dist_dir: union/dataset_coco/test
    :param img_index:
    :return:
    """
    anno_new = []
    img_new = []
    categories_new = []
    info_new = {}
    anno_id = 0
    for data_dir in dataset_dirs:
        annotations = {}
        label_json = data_dir.joinpath("result.json")
        with label_json.open("r", encoding="utf-8") as f:
            result_json = json.loads(f.read())
        # 图片
        _images = result_json["images"]
        # 标签类别
        _categories = result_json["categories"]
        categories_new = _categories
        # 标注
        _annotations = result_json["annotations"]
        # info
        _info = result_json["info"]
        info_new = _info
        # 遍历标签
        for anno in _annotations:
            image_id = anno["image_id"]
            if image_id not in annotations:
                annotations[image_id] = [anno]
            else:
                annotations[image_id].append(anno)
        img_dir = dist_dir.joinpath("images")
        if not img_dir.exists():
            img_dir.mkdir(parents=True)
        # 遍历图片
        for img in _images:
            file_name = img["file_name"]
            file_name = str(file_name).rsplit("/")[-1]
            img_file = data_dir.joinpath("images").joinpath(file_name)
            anno_list = annotations.get(img["id"])
            if not anno_list:
                continue
            # 修改标注的图片id
            for anno in anno_list:
                anno["image_id"] = img_index
                assert anno["category_id"] is not None
                anno["id"] = anno_id
                anno_id += 1
            img_new.append({"width": img.get("width"), "height": img.get("height"), "id": img_index,
                            "file_name": "{0}{1}".format(img_index, img_file.suffix)})
            anno_new.extend(annotations.get(img["id"]))
            copy_file_ = img_dir.joinpath("{0}{1}".format(img_index, img_file.suffix))
            shutil.copy(img_file, copy_file_)
            img_index += 1
            print(img_index, data_dir)
    if not dist_dir.exists():
        dist_dir.mkdir(parents=True)
    result_file = dist_dir.joinpath("result.json")
    with open(result_file, 'w') as f:
        json.dump({
            "images": img_new,
            "categories": categories_new,
            "annotations": anno_new,
            "info": info_new
        }, f, ensure_ascii=False, indent=4)
    pass


if __name__ == "__main__":
    pass




# -*- coding:utf-8 -*-
# Author:  zhousf
# Date:    2022-10-21
# Description: 并发测试工具
# pip install locust
import os
import time
from locust import HttpUser, task, TaskSet

HOST = "http://127.0.0.1:5000"
API = "/api/demo"
TEST_NAME = "api测试"
TEST_IMAGE_PATH = r"/Users/zhousf/Desktop/tmp/1.png"


class TestTask(TaskSet):

    def on_start(self):
        print("********** 测试开始 **********")

    @task
    def run_test(self):
        client = "user"
        req_id = str(time.time())
        params = {
            "client": client,
            "reqId": req_id
        }
        files = [
            ("images", open(TEST_IMAGE_PATH, 'rb'))
        ]
        with self.client.request(method="post", url="{0}/{1}".format(HOST, API), data=params,
                                 files=files, timeout=10, name=TEST_NAME,
                                 catch_response=True) as response:
            data = response.json()
            code = data.get("status", 408)
            if code == 200:
                print("请求成功", response.text)
                response.success()
            else:
                print("请求失败：{0}".format(response.text))
                response.failure(response.text)

    def on_stop(self):
        print("********** 测试结束 **********")


class User(HttpUser):
    tasks = [TestTask]
    host = HOST


if __name__ == '__main__':
    os.system("locust -f {0} --web-host=127.0.0.1".format(os.path.basename(__file__)))



# -*- coding: utf-8 -*-
# @Author  : zhousf
# @Function: 日期、时间工具
import time
import datetime
import cn2an
import re

CN_NUM = {'〇': '0', '一': '1', '二': '2', '三': '3', '四': '4', '五': '5', '六': '6', '七': '7', '八': '8', '九': '9',
          '零': '0', '壹': '1', '贰': '2', '叁': '3', '肆': '4', '伍': '5', '陆': '6', '柒': '7', '捌': '8', '玖': '9',
          '两': '2'}


def get_date_format(format_str):
    """
    获取日期
    :param format_str: "%Y-%m-%d %H:%M:%S"
    :return:
    """
    return time.strftime(format_str, time.localtime())


def get_date():
    """
    获取秒级日期
    :return: 2020-02-26 10:31:02
    """
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())


def get_date_ms():
    """
    获取毫秒级日期
    :return: 2021-08-20 14:46:43.805594
    """
    return datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')


def get_y_m_d():
    """
    获取日期
    :return: 2020_02_26
    """
    return time.strftime("%Y_%m_%d", time.localtime())


def get_timestamp_second():
    """
    秒级时间戳
    :return: 1582684262.161325
    """
    return time.time()


def get_timestamp_ms():
    """
    毫秒级时间戳
    :return: 1582684262161
    """
    return int(round(time.time() * 1000))


def get_timestamp_ws():
    """
    微秒级时间戳
    :return: 1582684262161369
    """
    return int(round(time.time() * 1000000))


def compare_time(time1: str, time2: str):
    """
    比较两个字符型日期大小 日期格式:%Y年%m月%d日
    :param time1: 第1个时间
    :param time2: 第2个时间
    :return: 整数  <0 time1 早于 time2  ;  =0 time1 = time2 ; >0 time1 晚于 time2
    """
    def trans_cn2d(cn_str):
        return ''.join([CN_NUM[i] if i in CN_NUM else i for i in cn_str])

    if re.match(r'(\d{4}-\d{1,2}-\d{1,2})',time1):
        format_str = '%Y-%m-%d'
    elif re.match(r'(\d{4}年\d{1,2}月\d{1,2}日)',time1):
        format_str = '%Y年%m月%d日'
    elif re.match(r'(\d{4}/\d{1,2}/\d{1,2})',time1):
        format_str = '%Y/%m/%d'
    elif re.match(r'(.{4}年.{1,2}月.{1,3}日)',time1):
        format_str = 'CN年CN月CN日'
    else:
        format_str = '%Y-%m-%d'

    if format_str == 'CN年CN月CN日':

        time1_cn = trans_cn2d(time1[:time1.index('年')]) \
                   + time1[time1.index('年'):time1.index('年') + 1] + \
                   str(cn2an.cn2an(time1[time1.index('年') + 1:time1.index('月')], 'normal')) \
                   + time1[time1.index('月'):time1.index('月') + 1] + \
                   str(cn2an.cn2an(time1[time1.index('月') + 1:time1.index('日')], 'normal')) \
                   + time1[time1.index('日'):time1.index('日') + 1]
        time2_cn = trans_cn2d((time2[:time2.index('年')])) \
                   + time2[time2.index('年'):time2.index('年') + 1] + \
                   str(cn2an.cn2an(time2[time2.index('年') + 1:time2.index('月')], 'normal')) \
                   + time2[time2.index('月'):time2.index('月') + 1] + \
                   str(cn2an.cn2an(time2[time2.index('月') + 1:time2.index('日')], 'normal')) \
                   + time2[time2.index('日'):time2.index('日') + 1]
        # print(time1_cn,time2_cn)
        s_time = time.mktime(time.strptime(time1_cn, '%Y年%m月%d日'))
        e_time = time.mktime(time.strptime(time2_cn, '%Y年%m月%d日'))
        if int(s_time) == int(e_time):
            return 1
        return int(s_time) - int(e_time)

    s_time = time.mktime(time.strptime(time1, format_str))
    e_time = time.mktime(time.strptime(time2, format_str))
    # print('s_time is:', s_time)
    # print('e_time is:', e_time)
    if int(s_time) == int(e_time):
        return 1
    return int(s_time) - int(e_time)


if __name__ == '__main__':
    print(compare_time('2019年02月03日', '2019年02月04日'))
    print(compare_time('二0一6年一月十日', '2〇1七年七月十七日'))
    print(compare_time('二〇一六年一月十日', '二〇一七年七月十七日'))
    print(compare_time('2021年12月31日', '2022年12月31日'))
    print(compare_time('2021-12-31', '2021-12-19'))
    print(compare_time('2021/12/31', '2021/12/19'))
    print(get_date_format("%Y%m%d%H%M%S"))
    print(get_date())
    print(get_date_ms())
    print(get_y_m_d())
    print(get_timestamp_second())
    print(get_timestamp_second())
    print(get_timestamp_ms())
    print(get_timestamp_ms())
    print(get_timestamp_ws())
    print(get_timestamp_ws())
    pass
